@@ -7,7 +7,6 @@ import json
 import re
 import threading
 import warnings
-from abc import abstractmethod
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from pyproj._crs import (
@@ -140,6 +139,16 @@ def _prepare_from_epsg(auth_code: Union[str, int]):
     return _prepare_from_authority("epsg", auth_code)
 
 
+def _is_epsg_code(auth_code: Any) -> bool:
+    if isinstance(auth_code, int):
+        return True
+    if isinstance(auth_code, str) and auth_code.isnumeric():
+        return True
+    if hasattr(auth_code, "shape") and auth_code.shape == ():
+        return True
+    return False
+
+
 class CRS:
     """
     A pythonic Coordinate Reference System manager.
@@ -301,12 +310,12 @@ class CRS:
         if projparams:
             if isinstance(projparams, _CRS):
                 projstring = projparams.srs
+            elif _is_epsg_code(projparams):
+                projstring = _prepare_from_epsg(projparams)
             elif isinstance(projparams, str):
                 projstring = _prepare_from_string(projparams)
             elif isinstance(projparams, dict):
                 projstring = _prepare_from_dict(projparams)
-            elif isinstance(projparams, int):
-                projstring = _prepare_from_epsg(projparams)
             elif isinstance(projparams, (list, tuple)) and len(projparams) == 2:
                 projstring = _prepare_from_authority(*projparams)
             elif hasattr(projparams, "to_wkt"):
@@ -969,7 +978,7 @@ class CRS:
         Returns
         -------
         CRS:
-            The the geodeticCRS / geographicCRS from the CRS.
+            The geodeticCRS / geographicCRS from the CRS.
 
         """
         return (
@@ -981,7 +990,7 @@ class CRS:
     @property
     def source_crs(self) -> Optional["CRS"]:
         """
-        The the base CRS of a BoundCRS or a DerivedCRS/ProjectedCRS,
+        The base CRS of a BoundCRS or a DerivedCRS/ProjectedCRS,
         or the source CRS of a CoordinateOperation.
 
         Returns
@@ -1201,7 +1210,16 @@ class CRS:
         -------
         str
         """
-        return self._crs.to_wkt(version=version, pretty=pretty)
+        wkt = self._crs.to_wkt(version=version, pretty=pretty)
+        if wkt is None:
+            warnings.warn(
+                f"CRS cannot be converted to a WKT string of a '{version}' version. "
+                "Select a different version of a WKT string or edit your CRS. Future"
+                "versions of pyproj will raise a CRSError in this case.",
+                FutureWarning,
+                stacklevel=2,
+            )
+        return wkt
 
     def to_json(self, pretty: bool = False, indentation: int = 2) -> str:
         """
@@ -1220,7 +1238,15 @@ class CRS:
         -------
         str
         """
-        return self._crs.to_json(pretty=pretty, indentation=indentation)
+        proj_json = self._crs.to_json(pretty=pretty, indentation=indentation)
+        if proj_json is None:
+            warnings.warn(
+                "CRS cannot be converted to a PROJ JSON string. "
+                "Future versions of pyproj will raise a CRSError in this case.",
+                FutureWarning,
+                stacklevel=2,
+            )
+        return proj_json
 
     def to_json_dict(self) -> dict:
         """
@@ -1253,7 +1279,15 @@ class CRS:
         -------
         str
         """
-        return self._crs.to_proj4(version=version)
+        proj = self._crs.to_proj4(version=version)
+        if proj is None:
+            warnings.warn(
+                "CRS cannot be converted to a PROJ string. "
+                "Future versions of pyproj will raise a CRSError in this case.",
+                FutureWarning,
+                stacklevel=2,
+            )
+        return proj
 
     def to_epsg(self, min_confidence: int = 70) -> Optional[int]:
         """
@@ -1591,7 +1625,6 @@ class CustomConstructorCRS(CRS):
     """
 
     @property
-    @abstractmethod
     def _expected_types(self) -> Tuple[str, ...]:
         """
         These are the type names of the CRS class
@@ -1648,7 +1681,7 @@ class CustomConstructorCRS(CRS):
         Returns
         -------
         CRS:
-            The the geodeticCRS / geographicCRS from the CRS.
+            The geodeticCRS / geographicCRS from the CRS.
 
         """
         return None if self._crs.geodetic_crs is None else CRS(self._crs.geodetic_crs)
@@ -1656,7 +1689,7 @@ class CustomConstructorCRS(CRS):
     @property
     def source_crs(self) -> Optional["CRS"]:
         """
-        The the base CRS of a BoundCRS or a DerivedCRS/ProjectedCRS,
+        The base CRS of a BoundCRS or a DerivedCRS/ProjectedCRS,
         or the source CRS of a CoordinateOperation.
 
         Returns
