@@ -1,7 +1,6 @@
 """
 The transformer module is for performing cartographic transformations.
 """
-
 __all__ = [
     "transform",
     "itransform",
@@ -16,17 +15,17 @@ from array import array
 from dataclasses import dataclass
 from itertools import chain, islice
 from pathlib import Path
-from typing import Any, Iterable, Iterator, List, Optional, Tuple, Union
+from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, Union, overload
 
 from pyproj import CRS
+from pyproj._compat import cstrencode
 from pyproj._crs import AreaOfUse, CoordinateOperation
-from pyproj._transformer import (  # noqa
+from pyproj._transformer import (  # noqa: F401 pylint: disable=unused-import
     AreaOfInterest,
     _Transformer,
     _TransformerGroup,
     proj_version_str,
 )
-from pyproj.compat import cstrencode
 from pyproj.datadir import get_user_data_dir
 from pyproj.enums import ProjVersion, TransformDirection, WktVersion
 from pyproj.exceptions import ProjError
@@ -81,12 +80,12 @@ class TransformerFromCRS(TransformerMaker):
     Generates a Cython _Transformer class from input CRS data.
     """
 
-    crs_from: str
-    crs_to: str
+    crs_from: bytes
+    crs_to: bytes
     always_xy: bool
     area_of_interest: Optional[AreaOfInterest]
     authority: Optional[str]
-    accuracy: Optional[float]
+    accuracy: Optional[str]
     allow_ballpark: Optional[bool]
 
     def __call__(self) -> _Transformer:
@@ -96,8 +95,8 @@ class TransformerFromCRS(TransformerMaker):
         _Transformer
         """
         return _Transformer.from_crs(
-            cstrencode(self.crs_from),
-            cstrencode(self.crs_to),
+            self.crs_from,
+            self.crs_to,
             always_xy=self.always_xy,
             area_of_interest=self.area_of_interest,
             authority=self.authority,
@@ -114,7 +113,7 @@ class TransformerFromPipeline(TransformerMaker):
     Generates a Cython _Transformer class from input pipeline data.
     """
 
-    proj_pipeline: str
+    proj_pipeline: bytes
 
     def __call__(self) -> _Transformer:
         """
@@ -122,7 +121,7 @@ class TransformerFromPipeline(TransformerMaker):
         -------
         _Transformer
         """
-        return _Transformer.from_pipeline(cstrencode(self.proj_pipeline))
+        return _Transformer.from_pipeline(self.proj_pipeline)
 
 
 class TransformerGroup(_TransformerGroup):
@@ -163,14 +162,13 @@ class TransformerGroup(_TransformerGroup):
             Projection of input data.
         crs_to: pyproj.crs.CRS or input used to create one
             Projection of output data.
-        skip_equivalent: bool, optional
+        skip_equivalent: bool, default=False
             DEPRECATED: If true, will skip the transformation operation
-            if input and output projections are equivalent. Default is false.
-        always_xy: bool, optional
+            if input and output projections are equivalent.
+        always_xy: bool, default=False
             If true, the transform method will accept as input and return as output
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
-            Default is false.
         area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
             The area of interest to help order the transformations based on the
             best operation for the area.
@@ -190,6 +188,7 @@ class TransformerGroup(_TransformerGroup):
             area_of_interest=area_of_interest,
         )
         for iii, transformer in enumerate(self._transformers):
+            # pylint: disable=unsupported-assignment-operation
             self._transformers[iii] = Transformer(TransformerUnsafe(transformer))
 
     @property
@@ -238,15 +237,14 @@ class TransformerGroup(_TransformerGroup):
         directory: str or Path, optional
             The directory to download the grids to.
             Defaults to :func:`pyproj.datadir.get_user_data_dir`
-        open_license: bool, optional
+        open_license: bool, default=True
             If True, will only download grids with an open license.
-            Defaults to False.
-        verbose: bool, optional
+        verbose: bool, default=False
             If True, will print information about grids downloaded.
-            Default is False.
         """
         if directory is None:
             directory = get_user_data_dir(True)
+        # pylint: disable=not-an-iterable
         for unavailable_operation in self.unavailable_operations:
             for grid in unavailable_operation.grids:
                 if (
@@ -282,6 +280,7 @@ class TransformerLocal(threading.local):
 
     def __init__(self):
         self.transformer = None  # Initialises in each thread
+        super().__init__()
 
 
 class Transformer:
@@ -311,6 +310,14 @@ class Transformer:
         self._local.transformer = transformer_maker()
         self._transformer_maker = transformer_maker
 
+    def __getstate__(self) -> Dict[str, Any]:
+        return {"_transformer_maker": self._transformer_maker}
+
+    def __setstate__(self, state: Dict[str, Any]):
+        self.__dict__.update(state)
+        self._local = TransformerLocal()
+        self._local.transformer = self._transformer_maker()
+
     @property
     def _transformer(self):
         """
@@ -417,6 +424,38 @@ class Transformer:
         """
         return self._transformer.is_network_enabled
 
+    @property
+    def source_crs(self) -> Optional[CRS]:
+        """
+        .. versionadded:: 3.3.0
+
+        Returns
+        -------
+        Optional[CRS]:
+            The source CRS of a CoordinateOperation.
+        """
+        return (
+            None
+            if self._transformer.source_crs is None
+            else CRS(self._transformer.source_crs)
+        )
+
+    @property
+    def target_crs(self) -> Optional[CRS]:
+        """
+        .. versionadded:: 3.3.0
+
+        Returns
+        -------
+        Optional[CRS]:
+            The target CRS of a CoordinateOperation.
+        """
+        return (
+            None
+            if self._transformer.target_crs is None
+            else CRS(self._transformer.target_crs)
+        )
+
     @staticmethod
     def from_proj(
         proj_from: Any,
@@ -438,14 +477,13 @@ class Transformer:
             Projection of input data.
         proj_to: :obj:`pyproj.Proj` or input used to create one
             Projection of output data.
-        skip_equivalent: bool, optional
+        skip_equivalent: bool, default=False
             DEPRECATED: If true, will skip the transformation operation
-            if input and output projections are equivalent. Default is false.
-        always_xy: bool, optional
+            if input and output projections are equivalent.
+        always_xy: bool, default=False
             If true, the transform method will accept as input and return as output
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
-            Default is false.
         area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
             The area of interest to help select the transformation.
 
@@ -454,6 +492,7 @@ class Transformer:
         Transformer
 
         """
+        # pylint: disable=import-outside-toplevel
         from pyproj import Proj
 
         if not isinstance(proj_from, Proj):
@@ -494,14 +533,13 @@ class Transformer:
             Projection of input data.
         crs_to: pyproj.crs.CRS or input used to create one
             Projection of output data.
-        skip_equivalent: bool, optional
+        skip_equivalent: bool, default=False
             DEPRECATED: If true, will skip the transformation operation
-            if input and output projections are equivalent. Default is false.
-        always_xy: bool, optional
+            if input and output projections are equivalent.
+        always_xy: bool, default=False
             If true, the transform method will accept as input and return as output
             coordinates using the traditional GIS order, that is longitude, latitude
             for geographic CRS and easting, northing for most projected CRS.
-            Default is false.
         area_of_interest: :class:`pyproj.transformer.AreaOfInterest`, optional
             The area of interest to help select the transformation.
         authority: str, optional
@@ -512,14 +550,13 @@ class Transformer:
             to “any”, then coordinate operations from any authority will be
             searched. If authority is a non-empty string different from "any",
             then coordinate operations will be searched only in that authority
-            namespace (e.g. EPSG). Requires PROJ 8+.
+            namespace (e.g. EPSG).
         accuracy: float, optional
             The minimum desired accuracy (in metres) of the candidate
-            coordinate operations. Requires PROJ 8+.
+            coordinate operations.
         allow_ballpark: bool, optional
             Set to False to disallow the use of Ballpark transformation
             in the candidate coordinate operations. Default is to allow.
-            Requires PROJ 8+.
 
         Returns
         -------
@@ -535,12 +572,12 @@ class Transformer:
 
         return Transformer(
             TransformerFromCRS(
-                CRS.from_user_input(crs_from).srs,
-                CRS.from_user_input(crs_to).srs,
+                cstrencode(CRS.from_user_input(crs_from).srs),
+                cstrencode(CRS.from_user_input(crs_to).srs),
                 always_xy=always_xy,
                 area_of_interest=area_of_interest,
                 authority=authority,
-                accuracy=accuracy,
+                accuracy=accuracy if accuracy is None else str(accuracy),
                 allow_ballpark=allow_ballpark,
             )
         )
@@ -549,7 +586,7 @@ class Transformer:
     def from_pipeline(proj_pipeline: str) -> "Transformer":
         """Make a Transformer from a PROJ pipeline string.
 
-        https://proj.org/operations/pipeline.html
+        :ref:`pipeline`
 
         .. versionadded:: 3.1.0 AUTH:CODE string suppor (e.g. EPSG:1671)
 
@@ -576,23 +613,64 @@ class Transformer:
         Transformer
 
         """
-        return Transformer(TransformerFromPipeline(proj_pipeline))
+        return Transformer(TransformerFromPipeline(cstrencode(proj_pipeline)))
 
-    def transform(
+    @overload
+    def transform(  # pylint: disable=invalid-name
         self,
         xx: Any,
         yy: Any,
-        zz: Any = None,
-        tt: Any = None,
         radians: bool = False,
         errcheck: bool = False,
         direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
-    ) -> Any:
+        inplace: bool = False,
+    ) -> Tuple[Any, Any]:
+        ...
+
+    @overload
+    def transform(  # pylint: disable=invalid-name
+        self,
+        xx: Any,
+        yy: Any,
+        zz: Any,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+        inplace: bool = False,
+    ) -> Tuple[Any, Any, Any]:
+        ...
+
+    @overload
+    def transform(  # pylint: disable=invalid-name
+        self,
+        xx: Any,
+        yy: Any,
+        zz: Any,
+        tt: Any,
+        radians: bool = False,
+        errcheck: bool = False,
+        direction: Union[TransformDirection, str] = TransformDirection.FORWARD,
+        inplace: bool = False,
+    ) -> Tuple[Any, Any, Any, Any]:
+        ...
+
+    def transform(  # pylint: disable=invalid-name
+        self,
+        xx,
+        yy,
+        zz=None,
+        tt=None,
+        radians=False,
+        errcheck=False,
+        direction=TransformDirection.FORWARD,
+        inplace=False,
+    ):
         """
         Transform points between two coordinate systems.
 
         .. versionadded:: 2.1.1 errcheck
         .. versionadded:: 2.2.0 direction
+        .. versionadded:: 3.2.0 inplace
 
         Parameters
         ----------
@@ -604,20 +682,24 @@ class Transformer:
             Input z coordinate(s).
         tt: scalar or array (numpy or python), optional
             Input time coordinate(s).
-        radians: boolean, optional
+        radians: bool, default=False
             If True, will expect input data to be in radians and will return radians
-            if the projection is geographic. Default is False (degrees). Ignored for
-            pipeline transformations with pyproj 2, but will work in pyproj 3.
-        errcheck: boolean, optional
-            If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation
-            returns ``inf`` and no exception is raised.
+            if the projection is geographic. Otherwise, it uses degrees.
+            Ignored for pipeline transformations with pyproj 2,
+            but will work in pyproj 3.
+        errcheck: bool, default=False
+            If True, an exception is raised if the errors are found in the process.
+            If False, ``inf`` is returned for errors.
         direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
             Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
+        inplace: bool, default=False
+            If True, will attempt to write the results to the input array
+            instead of returning a new array. This will fail if the input
+            is not an array in C order with the double data type.
 
-
-        Example:
+        Example
+        --------
 
         >>> from pyproj import Transformer
         >>> transformer = Transformer.from_crs("epsg:4326", "epsg:3857")
@@ -660,14 +742,14 @@ class Transformer:
 
         """
         # process inputs, making copies that support buffer API.
-        inx, xisfloat, xislist, xistuple = _copytobuffer(xx)
-        iny, yisfloat, yislist, yistuple = _copytobuffer(yy)
+        inx, x_data_type = _copytobuffer(xx, inplace=inplace)
+        iny, y_data_type = _copytobuffer(yy, inplace=inplace)
         if zz is not None:
-            inz, zisfloat, zislist, zistuple = _copytobuffer(zz)
+            inz, z_data_type = _copytobuffer(zz, inplace=inplace)
         else:
             inz = None
         if tt is not None:
-            intime, tisfloat, tislist, tistuple = _copytobuffer(tt)
+            intime, t_data_type = _copytobuffer(tt, inplace=inplace)
         else:
             intime = None
         # call pj_transform.  inx,iny,inz buffers modified in place.
@@ -681,17 +763,13 @@ class Transformer:
             errcheck=errcheck,
         )
         # if inputs were lists, tuples or floats, convert back.
-        outx = _convertback(xisfloat, xislist, xistuple, inx)
-        outy = _convertback(yisfloat, yislist, yistuple, iny)
-        return_data = (outx, outy)
+        outx = _convertback(x_data_type, inx)
+        outy = _convertback(y_data_type, iny)
+        return_data: Tuple[Any, ...] = (outx, outy)
         if inz is not None:
-            return_data += (  # type: ignore
-                _convertback(zisfloat, zislist, zistuple, inz),
-            )
+            return_data += (_convertback(z_data_type, inz),)
         if intime is not None:
-            return_data += (  # type: ignore
-                _convertback(tisfloat, tislist, tistuple, intime),
-            )
+            return_data += (_convertback(t_data_type, intime),)
         return return_data
 
     def itransform(
@@ -713,25 +791,26 @@ class Transformer:
         ----------
         points: list
             List of point tuples.
-        switch: boolean, optional
+        switch: bool, default=False
             If True x, y or lon,lat coordinates of points are switched to y, x
             or lat, lon. Default is False.
-        time_3rd: boolean, optional
+        time_3rd: bool, default=False
             If the input coordinates are 3 dimensional and the 3rd dimension is time.
-        radians: boolean, optional
+        radians: bool, default=False
             If True, will expect input data to be in radians and will return radians
-            if the projection is geographic. Default is False (degrees). Ignored for
-            pipeline transformations.
-        errcheck: boolean, optional
-            If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation
-            returns ``inf`` and no exception is raised.
+            if the projection is geographic. Otherwise, it uses degrees.
+            Ignored for pipeline transformations with pyproj 2,
+            but will work in pyproj 3.
+        errcheck: bool, default=False
+            If True, an exception is raised if the errors are found in the process.
+            If False, ``inf`` is returned for errors.
         direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
             Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
 
 
-        Example:
+        Example
+        --------
 
         >>> from pyproj import Transformer
         >>> transformer = Transformer.from_crs(4326, 2100)
@@ -780,12 +859,12 @@ class Transformer:
         '-2.137 0.661'
 
         """
-        it = iter(points)  # point iterator
+        point_it = iter(points)  # point iterator
         # get first point to check stride
         try:
-            fst_pt = next(it)
+            fst_pt = next(point_it)
         except StopIteration:
-            raise ValueError("iterable must contain at least one point")
+            raise ValueError("iterable must contain at least one point") from None
 
         stride = len(fst_pt)
         if stride not in (2, 3, 4):
@@ -796,7 +875,9 @@ class Transformer:
 
         # create a coordinate sequence generator etc. x1,y1,z1,x2,y2,z2,....
         # chain so the generator returns the first point that was already acquired
-        coord_gen = chain(fst_pt, (coords[c] for coords in it for c in range(stride)))
+        coord_gen = chain(
+            fst_pt, (coords[c] for coords in point_it for c in range(stride))
+        )
 
         while True:
             # create a temporary buffer storage for
@@ -815,8 +896,8 @@ class Transformer:
                 errcheck=errcheck,
             )
 
-            for pt in zip(*([iter(buff)] * stride)):
-                yield pt
+            for point in zip(*([iter(buff)] * stride)):
+                yield point
 
     def transform_bounds(
         self,
@@ -835,28 +916,49 @@ class Transformer:
         Transform boundary densifying the edges to account for nonlinear
         transformations along these edges and extracting the outermost bounds.
 
+        If the destination CRS is geographic and right < left then the bounds
+        crossed the antimeridian. In this scenario there are two polygons,
+        one on each side of the antimeridian. The first polygon should be
+        constructed with (left, bottom, 180, top) and the second with
+        (-180, bottom, top, right).
+
+        To construct the bounding polygons with shapely::
+
+            def bounding_polygon(left, bottom, right, top):
+                if right < left:
+                    return shapely.geometry.MultiPolygon(
+                        [
+                            shapely.geometry.box(left, bottom, 180, top),
+                            shapely.geometry.box(-180, bottom, right, top),
+                        ]
+                    )
+                return shapely.geometry.box(left, bottom, right, top)
+
+
         Parameters
         ----------
         left: float
-            Left bounding coordinate in source CRS.
+            Minimum bounding coordinate of the first axis in source CRS
+            (or the target CRS if using the reverse direction).
         bottom: float
-            Bottom bounding coordinate in source CRS.
+            Minimum bounding coordinate of the second axis in source CRS.
+            (or the target CRS if using the reverse direction).
         right: float
-            Right bounding coordinate in source CRS.
+            Maximum bounding coordinate of the first axis in source CRS.
+            (or the target CRS if using the reverse direction).
         top: float
-            Top bounding coordinate in source CRS.
-        densify_points: uint, optional
+            Maximum bounding coordinate of the second axis in source CRS.
+            (or the target CRS if using the reverse direction).
+        densify_points: uint, default=21
             Number of points to add to each edge to account for nonlinear edges
             produced by the transform process. Large numbers will produce worse
-            performance. Default: 21 (gdal default).
-        radians: boolean, optional
+            performance.
+        radians: bool, default=False
             If True, will expect input data to be in radians and will return radians
-            if the projection is geographic. Default is False (degrees). Ignored for
-            pipeline transformations.
-        errcheck: boolean, optional
-            If True an exception is raised if the transformation is invalid.
-            By default errcheck=False and an invalid transformation
-            returns ``inf`` and no exception is raised.
+            if the projection is geographic. Otherwise, it uses degrees.
+        errcheck: bool, default=False
+            If True, an exception is raised if the errors are found in the process.
+            If False, ``inf`` is returned for errors.
         direction: pyproj.enums.TransformDirection, optional
             The direction of the transform.
             Default is :attr:`pyproj.enums.TransformDirection.FORWARD`.
@@ -892,8 +994,8 @@ class Transformer:
         version: pyproj.enums.ProjVersion
             The version of the PROJ string output.
             Default is :attr:`pyproj.enums.ProjVersion.PROJ_5`.
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
 
         Returns
         -------
@@ -922,11 +1024,11 @@ class Transformer:
 
         Parameters
         ----------
-        version: pyproj.enums.WktVersion
+        version: pyproj.enums.WktVersion, optional
             The version of the WKT output.
             Default is :attr:`pyproj.enums.WktVersion.WKT2_2019`.
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
 
         Returns
         -------
@@ -943,10 +1045,10 @@ class Transformer:
 
         Parameters
         ----------
-        pretty: bool
-            If True, it will set the output to be a multiline string. Defaults to False.
-        indentation: int
-            If pretty is True, it will set the width of the indentation. Default is 2.
+        pretty: bool, default=False
+            If True, it will set the output to be a multiline string.
+        indentation: int, default=2
+            If pretty is True, it will set the width of the indentation.
 
         Returns
         -------
@@ -984,12 +1086,24 @@ class Transformer:
         return self._transformer.__eq__(other._transformer)
 
     def is_exact_same(self, other: Any) -> bool:
+        """
+        Check if the Transformer objects are the exact same.
+        If it is not a Transformer, then it returns False.
+
+        Parameters
+        ----------
+        other: Any
+
+        Returns
+        -------
+        bool
+        """
         if not isinstance(other, Transformer):
             return False
         return self._transformer.is_exact_same(other._transformer)
 
 
-def transform(
+def transform(  # pylint: disable=invalid-name
     p1: Any,
     p2: Any,
     x: Any,
@@ -1090,7 +1204,7 @@ def transform(
     ).transform(xx=x, yy=y, zz=z, tt=tt, radians=radians, errcheck=errcheck)
 
 
-def itransform(
+def itransform(  # pylint: disable=invalid-name
     p1: Any,
     p2: Any,
     points: Iterable[Iterable],
